from dataclasses import dataclass
from injector import inject
from lmoe.api.lmoe_query import LmoeQuery
from lmoe.commands.command import Command
from lmoe.experts.classifier import Classifier

import argparse


@inject
@dataclass
class PrintClassification(Command):

    classifier: Classifier

    def execute(self, parsed_args: argparse.Namespace, lmoe_query: LmoeQuery) -> None:
        print(self.classifier.classify(lmoe_query))



import importlib.metadata
import importlib.resources
import os


def get_template_module():
    """The module string within which all of the template resource files reside."""
    return "lmoe.templates"


def get_project_version():
    """Returns the semantic versioning string associated with this project."""
    return importlib.metadata.version("lmoe")


# The absolute filenames of all resource template files
_TEMPLATE_FILE_NAMES = None


def get_template_file_names():
    global _TEMPLATE_FILE_NAMES
    if _TEMPLATE_FILE_NAMES is None:
        template_dir_object = importlib.resources.files(get_template_module())
        _TEMPLATE_FILE_NAMES = [
            os.path.join(template_dir_object.name, file_object.name)
            for file_object in template_dir_object.iterdir()
        ]
    return _TEMPLATE_FILE_NAMES



from lmoe.api.model_expert import ModelExpert
from lmoe.api.lmoe_query import LmoeQuery
from lmoe.api.ollama_client import stream
from lmoe.framework.expert_registry import expert


@expert
class Code(ModelExpert):

    @classmethod
    def name(cls):
        return "CODE"

    def description(self):
        return "A model specifically for generating code. It is expected that this is an instruction-tuned model rather than a 'fill in the middle' model, meaning that a user will describe a coding task or coding question in natural language rather than supplying code and expecting the following code to be generated."

    def example_queries(self):
        return [
            "write a python script which determines the largest directory in my home environment",
        ]

    def generate(self, lmoe_query: LmoeQuery):
        for chunk in stream(self.model, lmoe_query):
            print(chunk, end="", flush=True)
        print("")



from dataclasses import dataclass
from lmoe.definitions import get_project_version
from lmoe.utils.templates import read_template
from typing import Optional


@dataclass
class Model:
    """A basic Ollama model with a tuned modelfile.

    The modelfile is located in the templates directory at {name}.modelfile.txt and contains no
    additional templating such as examples, descriptions, etc.
    """

    name: str
    """The name of the model."""

    def ollama_name(self) -> str:
        """The name of this model in Ollama, a string which includes a namespace prefix and version."""
        return f"lmoe_{self.name.lower()}_{get_project_version()}"

    def modelfile_name(self) -> str:
        """The filename (without a path) of the backing Ollama model.

        `lmoe` convention is to use the model name suffixed with '.modelfile.txt':
          * code.modelfile.txt
          * general.modelfile.txt
          * etc.
        """
        return f"{self.name.lower()}.modelfile.txt"

    def modelfile_contents(self) -> Optional[str]:
        """The contents of the modelfile.

        May be overridden for dynamically generated modelfiles."""
        return read_template(self.modelfile_name())
